from pyparsing import Literal, Word, ZeroOrMore, Group, Dict, Optional,\
                      restOfLine, empty, printables, OneOrMore, oneOf, nums,\
                      lineno, line, col, Keyword, SkipTo, LineEnd, Combine
from enum import Enum


class CalculateIniParser:
    '''Класс парсера calculate.ini файлов.'''
    class Define(Enum):
        Assign = 0
        Append = 1
        Remove = 2

    def __init__(self):
        lbrack = Literal("[")
        rbrack = Literal("]")

        # comma = Literal(",").suppress()
        comment_symbol = Literal(';') | Literal('#')

        # Define = self.Define
        value_operation = (Literal("=") | Combine(Literal("+") + Literal("="))
                           | Combine(Literal("-") + Literal("=")))

        comment = comment_symbol + Optional(restOfLine)

        section_name = Word(printables+'\t', excludeChars='[]')

        value_name = Word(printables+'\t', excludeChars='=-+')

        # non_comma = Word(printables+'\t', excludeChars=',')
        clear_section = lbrack.suppress() + Group(empty) + rbrack.suppress()

        section_start = Group(OneOrMore(lbrack.suppress() + section_name
                              + rbrack.suppress())
                              + (clear_section | ~lbrack())
                              + LineEnd().suppress())

        # Если содержимое ini-файла не предваряется заголовком секции,
        # значит эта строка ошибочна.
        unexpected = Group(~section_start + SkipTo(LineEnd(),
                                                   include=True))("error")
        unexpected.setParseAction(self._unexpected_token)

        key_value = (~lbrack + value_name
                     + value_operation + empty
                     + restOfLine + LineEnd().suppress())

        def strip_key_value(tokens):
            tokens[0] = tokens[0].strip()
            tokens[1] = tokens[1].strip()

        key_value.setParseAction(strip_key_value)

        self.ini_section_parser = (section_start
                                   + Group(ZeroOrMore(
                                                Group(key_value | unexpected)))
                                   | unexpected)
        self.ini_section_parser.ignore(comment)

    def _unexpected_token(self, string, location, tokens):
        '''Метод вызываемый парсером, если обнаружена некорректная строка,
        предназначен для получения некорректной строки и ее дальнейшего
        разбора.'''
        error_line = line(location, string).strip()
        if error_line:
            self.parseError(error_line, lineno(location, string),
                            col(location, string))

    def parse(self, data):
        for tokens, start, end in self.ini_section_parser.scanString(data):
            if tokens.getName() == "error":
                continue
            section, defkeys = tokens
            sectionList = section.asList()
            if sectionList[-1] == []:
                self.clearSection(sectionList[:-1])
                continue
            self.startSection(section.asList())
            for defkey in defkeys:
                if defkey.getName() == "error":
                    continue
                mapOp = {"=": self.Define.Assign,
                         "+=": self.Define.Append,
                         "-=": self.Define.Remove}
                self.defineKey(section.asList(),
                               defkey[0], defkey[2],
                               mapOp[defkey[1]])

    def startSection(self, section):
        """Начало секции"""
        pass

    def clearSection(self, section):
        '''Метод для очистки секций.'''
        pass

    def defineKey(self, section, defkey, defval, deftype):
        '''Метод для определения ключа.
        ['section','block'], 'varname', 'varval', Define.Assign'''
        pass

    def parseError(self, line, lineno, col):
        '''Метод для обработки ошибок, обнаруженных в ini-файле.
        line - пример строки;
        lineno - номер строки;
        col - номер символа.'''
        pass
